#ifndef SPARTICLES_H
#define SPARTICLES_H


// Sparticles.h -- Interface for the Sparticles class
// Jon Pimble - fatrat@fat-rat.com


#include <stdlib.h>
#include <sys/types.h>
#include <time.h>
#include <math.h>
#include <string.h>

#include <ScreenSaver.h>
#include <Screen.h>
#include <Locker.h>

#include "Common_stuff.h"
#include "ConfigView.h"
#include "Vect2D.h"



// Class is derived from the BScreenSaver base class
class Sparticles : public BScreenSaver
{
  private:
    // Struct that defines a spark
    typedef struct
    {
      // Positions
      float x, y;

      // Colors
      unsigned char red, green, blue;

      // Trail behind us...
      int trail_x[MAX_TRAIL_LENGTH];
      int trail_y[MAX_TRAIL_LENGTH];
      int trail_ofs[MAX_TRAIL_LENGTH];
      int trail_counter;
      int trail_length;
  
      // When the spark will die and need to be renewed...
      int timeout;
  
      // Velocity vector
      Vect2D velocity;
  
    } spark;

    // Array of sparks
    spark sparks[MAX_SPARKS];
    
    // Initializes a spark
    void init_spark(int spark_num, float width, float height);


    // Configuration menu
    ConfigView *configs;
    
    // Values set through the config panel
    int32 num_sparks;
    int32 trail_length;
    float gravity;
    bool blending;

    // Copies of values set through the config panel, so we can test 'em and
    //  act if something important changed
    int32 old_num_sparks;
    int32 old_trail_length;
    float old_gravity;
    bool old_blending;

    // Stuff for DirectDraw rendering...
    char *pixel_data;
    int32 stride;
    uint32 bpp;
    color_space colorspace;
    clipping_rect dd_bounds;
    bool dd_mode;
    BLocker locker;
    
    // Color lookup tables to speed up and simplify rendering in
    //  non-RGB colorspaces
    // For 16bpp
    uint8 red_16_LUT[256];
    uint8 green_h_16_LUT[256];
    uint8 green_l_16_LUT[256];
    uint8 blue_16_LUT[256];
    // For 15bpp    
    uint8 red_15_LUT[256];
    uint8 green_h_15_LUT[256];
    uint8 green_l_15_LUT[256];
    // Blue component is the same in 16bpp and 15bpp, so we don't bother making
    //  an extra lookup table for it


  public:
    // Constructor
    Sparticles( BMessage *archive, image_id image );
    
    // Destructor 
    ~Sparticles( void ); 
    
    // Virtual function implementations from base class
    void StartConfig( BView *view ); 
    status_t SaveState(BMessage *into) const;
    status_t StartSaver( BView *view, bool preview ); 
    void Draw( BView *view, int32 frame );  
    void DirectConnected(direct_buffer_info *info);
};



#endif


