// Vect2D.cpp - 2D vector class implementation
// Jonathan Pimble - fatrat@fat-rat.com


#include "Vect2D.h"


// Member functions...

// Constructor
Vect2D::Vect2D(float new_x, float new_y)
{
  // Set vector components
  x = new_x;
  y = new_y;
}



// Returns the unit vector for this vector
Vect2D Vect2D::unit() const
{
  // Unit vector equals each component divided by the magnitude
  float tmp_mag = magnitude();

  // If the magnitude is 0, return the 0 vector
  if(tmp_mag == 0.0)
  {
    return Vect2D(0.0, 0.0);
  }
  else
  { 
    return Vect2D(x/tmp_mag, y/tmp_mag);
  }
}



// Returns the magnitude (i.e., length) of the vector
float Vect2D::magnitude() const 
{
  return sqrt((x*x) + (y*y));
}


// Overloaded operators

// Unary operator - negation
Vect2D Vect2D::operator-() const
{
  return Vect2D(-x, -y);
}

// Binary operator += addition
Vect2D & Vect2D::operator+=(const Vect2D & vector)
{
  return *this = *this + vector;
}

// Binary operator -= subtraction
Vect2D & Vect2D::operator-=(const Vect2D & vector)
{
  return *this = *this - vector;
}

// Binary operator *=  multiplication (by a scalar)  
Vect2D & Vect2D::operator*=(const float & factor)
{
  return *this = *this * factor;
}

// Binary operator /= division by a scalar (mult by 1/factor)
Vect2D & Vect2D::operator/=(const float & factor)
{
  return *this = *this / factor;
}


// Non-member functions...

// Overloaded operators
// Binary Comparison operator ==
bool operator ==(const Vect2D & a, const Vect2D & b)
{
  if( (a.x == b.x) && (a.y == b.y) )
  {
    return true;
  }
  else
  {
    return false;
  }
}

// Binary Comparison operator !=
bool operator !=(const Vect2D & a, const Vect2D & b)
{
  if( (a.x == b.x) && (a.y == b.y) )
  {
    return false;
  }
  else
  {
    return true;
  }
}


// Binary Addition operator +
Vect2D operator +(const Vect2D & a, const Vect2D & b)
{
  return Vect2D(a.x + b.x, a.y + b.y);
}

// Binary Subtraction operator -
Vect2D operator -(const Vect2D & a, const Vect2D & b)
{
  return Vect2D(a.x - b.x, a.y - b.y);
}


// Binary Multiplication operator *
Vect2D operator *(const Vect2D & vector, const float & factor)
{
  return Vect2D(vector.x * factor, vector.y * factor);
}

// Binary Division operator / (Multiply by 1/factor)
Vect2D operator /(const Vect2D & vector, const float & factor)
{
  if(factor != 0.0)
  {
    return Vect2D(vector.x / factor, vector.y / factor);
  }
  else
  {
    return Vect2D(0.0, 0.0);
  }
}



// Aux functionality

// Dot product - returns a scalar value
float dot_product(const Vect2D & a, const Vect2D & b)
{
  return ( (a.x * b.x) + (a.y * b.y) );
}



// Component of a onto b
float comp(const Vect2D & a, const Vect2D & b)
{
  return (dot_product(a, b) * a.magnitude());
}



// Projection of a onto b
Vect2D proj(const Vect2D & a, const Vect2D & b)
{
  return (a.unit() * comp(a, b) );
}



// Angle between a and b (in radians)
float angle(const Vect2D & a, const Vect2D & b)
{
  // If the magnitudes are both 0, then the angle between the vectors is
  //  also 0
  float tmp = a.magnitude() * b.magnitude();
  if(tmp == 0)
  {
    return 0.0;
  }
  else
  {
    return acos(dot_product(a, b)/tmp);
  }
}


