#ifndef VECT2D_H
#define VECT2D_H

// Vect2D.h -- 2D vector class interface
// Jonathan Pimble, fatrat@fat-rat.com


// This class provides some convenient functions for manipulating 2D vectors
//  Very straightforward, no fancy optimizations or caching


#include <math.h>


// 2D vector class interface
class Vect2D
{
  private:


  public:
    // No good reason to prevent user from accessing these directly   
    float x, y;
    
    // Constructor; default values for x, y
    Vect2D(float new_x = 0.0, float new_y = 0.0);
    

    // Member functions

    // Get the unit vector for this vector (i.e., vector with magnitude of 1.0)
    Vect2D unit() const;
    
    // Returns the magnitude of the vector (i.e., the length)
    float magnitude() const;    


    // Overloaded operators
    // Unary operator - negation
    Vect2D operator-() const;
    
    // Binary operator += addition
    Vect2D & operator+=(const Vect2D & vector);

    // Binary operator -= subtraction
    Vect2D & operator-=(const Vect2D & vector);
    
    // Binary operator *= multiplication (by a scalar)    
    Vect2D & operator*=(const float & factor);

    // Binary operator /= divison by a scalar (mult by 1/factor)    
    Vect2D & operator/=(const float & factor);
};


// Non-member functions

// Overloaded operators
// Binary comparison operator ==
bool operator ==(const Vect2D & a, const Vect2D & b);

// Binary comparison operator !=
bool operator !=(const Vect2D & a, const Vect2D & b);

// Binary addition operator +
Vect2D operator +(const Vect2D & a, const Vect2D & b);

// Binary subtraction operator -
Vect2D operator -(const Vect2D & a, const Vect2D & b);

// Binary multiplication by a scalar (operator *)
Vect2D operator *(const Vect2D & vector, const float & factor);
    
// Binary division by a scalar (mult by 1/scalar) (operator /)
Vect2D operator /(const Vect2D & vector, const float & factor);


// Aux functionality
// Dot product - returns a scalar value
float dot_product(const Vect2D & a, const Vect2D & b);

// Component of a onto b
float comp(const Vect2D & a, const Vect2D & b);

// Projection of a onto b
Vect2D proj(const Vect2D & a, const Vect2D & b);

// Angle between a and b (in radians)
float angle(const Vect2D & a, const Vect2D & b);



#endif



